/* here is the class definition for domain wall */
//
//
// 4/4/94 -- Upgraded to new formalism. gp.


#ifndef _DWM_DOMAIN_WALL_H
#define _DWM_DOMAIN_WALL_H

#include "dwm-object.h"

class DomainWall : public DWMObject {
public:
  
  DomainWall() { initialize(); }
  DomainWall(char *name) : DWMObject(name) { initialize(); }
  DomainWall( Vector Position, double radius, unsigned npoints, double initial_angle);
  ~DomainWall() { remove_all_wall_points(); }

  // here are the functions needed to make this an object 
  // derived from DWMObject..

  virtual DWMClassType Class_Type() const { return DWMDomainWallClass; }
  virtual char *Class_Name() const { return "DWMDomainWallClass"; }

  // ok. back to the business at hand..

  void Initialize_Bubble_Domain(Vector Position, double radius, unsigned npoints, double initial_angle);
  void Initialize_Domain_From_File();
  void Reset();

  unsigned Number_Of_Wall_Points() {return n_wall_points;}
  Vector  Domain_Position();
  double  Domain_Radius();
  double  Domain_Length();
  double  Domain_Length_From_Internals();
  double  Domain_Ellipsity() {return 0.0;}
  Vector  Maximum_Wall_Point_Displacement();

  /* here are iterator functions to make calls
     in the WallPoint data */

  void Step();
  void Examine_And_Correct_Shape();
  void Update_Fields();
  void Update_Internals();
  void Calculate_Demag_Field();
  void Calculate_Forces();
  void Calculate_Delta_R();
  void Calculate_Delta_S();
  void Calculate_Delta_Phia();
  void Calculate_Normalized_Distance();
  void Update_Position();

  /* some simple code to debug the domain wall structure */

  void Print_Diagnostics();
  void Print_Diagnostics_1();
  void Print_Diagnostics_2();
  void Print_Diagnostics_3();
  void Print_Fields();
  void Print_State_And_Phia(FILE *fp);
  void Print_Point_Diagnostics(int, int, FILE* );

  /* Eventually, we will need some code to read and write these structures.
   Put it here. */

  void Write(FILE *fp); 

  void allocate_wall_points(unsigned);
  void add_wall_point(WallPoint *wp1);
  void add_wall_point(int offset);
  void remove_wall_point(WallPoint *wp);
  void remove_all_wall_points();

  // These are for setting the starting bubble properties....
  void Set_Initial_X_Position(double x) { initial_x_position = x; }
  void Set_Initial_Y_Position(double y) { initial_y_position = y; }
  void Set_Initial_Radius(double     r) { initial_radius = r; }
  void Set_Initial_Npoints(unsigned    n) { initial_n_wall_points = n; }
  void Set_Initial_Winding_Number(int i)  { initial_winding_number = i; }
  void Set_Initial_State_Filename(char *name) { initial_state_filename = new char [ strlen(name)+1 ]; strcpy(initial_state_filename, name); }

  
private:

  WallPoint *wall_point_list;
  unsigned n_wall_points;

  // I also include some private variables to store the initial state of the
  // bubble so that it can easily be reset to its starting structure.

  double initial_x_position;
  double initial_y_position;
  double initial_radius;
  double initial_magnetization_angle;
  unsigned initial_n_wall_points;
  int      initial_winding_number;
  char *initial_state_filename;

  // here is a procedure to set these items up upon class instantiation
  void initialize() {
    wall_point_list = 0; n_wall_points = 0;
    initial_x_position = 0.0; initial_y_position = 0.0;
    initial_radius = 2000.0; initial_magnetization_angle = 90.0;
    initial_n_wall_points = 40; initial_winding_number = 1;
    initial_state_filename = 0;
  }
};

#endif


