//
// dwm-dfield.h
//
//
// 9/6/94 started by gp

#ifndef _DWM_GROOVE_FIELD_H
#define _DWM_GROOVE_FIELD_H

#include <assert.h>
#include "dwm-tfield.h"

class DWMGrooveField : public DWMTField {

public:
  DWMGrooveField() { initialize_with_defaults(); }
  ~DWMGrooveField() { remove_storage_space(); }

  // here are the functions that we must provide to make this
  // a concrete class.  The following are required by the ultimate base
  // base object DWMObject...

  virtual DWMClassType Class_Type() const { return DWMGrooveFieldClass; }
  virtual char *Class_Name() const { return "DWMGrooveFieldClass"; }

  //... and this function must be defined per the requirement in DWMField. 
  virtual double Evaluate_Field(double x, double y, double t)
    { return(Interpolate(x,y,0)); }

  double Interpolate(double x, double y, unsigned component);

  void Reset();

  /* Code to set the groove + associated parameters */
  /* Added 1/10/95 by gp in response to the problem associated with initializing
     multiple grooves */

  void Set_X_Center(double x) { groove_center_x = x; }
  void Set_Y_Center(double x) { groove_center_y = x; }

  void Set_X_Width(double x) { groove_width_x = x; }
  void Set_Y_Width(double x) { groove_width_y = x; }
  void Set_Z_Width(double x) { groove_width_z = x; }

  void Set_X_Lattice_Constant(double value) { lattice_constant_x = value; recompute_internals(); }
  void Set_Y_Lattice_Constant(double value) { lattice_constant_y = value; recompute_internals(); }

  void Set_X_Border(double x) { border_width_x = x; }
  void Set_Y_Border(double x) { border_width_y = x; }

  void Set_Rotation_Angle(double x) { groove_rotation_angle = x; }

  /* Code to get the lattice constant associated with the 
   X and Y axes respectively...*/

  double Lattice_Constant(unsigned axis) {
    double result;
    assert(axis == 0 || axis == 1);
    if(axis == 0) result=lattice_constant_x;
    if(axis == 1) result=lattice_constant_y;
    return result;
  }

  // code to get the origin of the lattice
  double Lattice_Origin(unsigned axis) {
    double result;
    assert(axis == 0 || axis == 1);
    if(axis == 0) result=lattice_origin_x;
    if(axis == 1) result=lattice_origin_y;
    return result;
  }

  // code to get the upper limits of the mesh along X and Y axes...

  double Lattice_Extent(unsigned axis) {
    double result;
    assert(axis == 0 || axis == 1);
    if(axis == 0) result=lattice_extent_x;
    if(axis == 1) result=lattice_extent_y;
    return result;
  }

  // Code to get the size of the lattice.  Intended to be used to
  //   do a read....

  unsigned Lattice_Size(unsigned axis) {
    unsigned result;
    assert(axis == 0 || axis == 1);
    if(axis == 0) result=number_of_cells_x;
    if(axis == 1) result=number_of_cells_y;
    return result;
  }

  // and of course some diagnostic code...

  void Print_Diagnostics();
  void Print_Data_Field();

  // and finally code to write a manhattan groove description

  void Write_Object_Description(FILE *);

private:

  void set_lattice_constant(unsigned axis, double value) { 
    assert(axis == 0 || axis == 1);
    if(axis == 0) lattice_constant_x = value; 
    if(axis == 1) lattice_constant_y = value;
    recompute_internals();
  }

  void set_lattice_constant(double x_value, double y_value) { 
    lattice_constant_x = x_value; 
    lattice_constant_y = y_value;
    recompute_internals();
  }

  void set_lattice_origin(unsigned axis, double value) { 
    assert(axis == 0 || axis == 1);
    if(axis == 0) lattice_origin_x = value; 
    if(axis == 1) lattice_origin_y = value;
    recompute_internals();
  }

  void set_lattice_origin(double x_value, double y_value) { 
    lattice_origin_x = x_value; 
    lattice_origin_y = y_value;
    recompute_internals();
  }

  void set_lattice_size(unsigned axis, unsigned value) { 
    assert(axis == 0 || axis == 1);
    if(axis == 0) number_of_cells_x = value; 
    if(axis == 1) number_of_cells_y = value;
  }

  void initialize_with_defaults();
  void reset();
  void allocate_storage_space();
  void remove_storage_space();
  void recompute_internals();
  double& map_node(unsigned x, unsigned y, unsigned nc);

  // Here are the physical properties of the groove...
  // All distance units are in nm....

  double groove_center_x;
  double groove_center_y;
  double groove_width_x;
  double groove_width_y;
  double groove_width_z;

  // These controll how far out to evaluate the field around the
  // groove.  They will be set by default if they are not specified by
  // the user. Added 1/10/95 by gnp

  double border_width_x;
  double border_width_y;

  // This controlls the rotation of the groove or ridge about its
  // origin.

  double groove_rotation_angle;

  // Lattice constants (and their product) for the 
  // regular array of numbers that is stored in data.

  double lattice_constant_x;
  double lattice_constant_y;
  double lattice_constant_product;

  // Origin (x and y) and extent of the mesh.
  double lattice_origin_x;
  double lattice_origin_y;
  double lattice_extent_x;
  double lattice_extent_y;

  // Number of cells and components..
  unsigned number_of_cells_x;
  unsigned number_of_cells_y;
  unsigned number_of_components;

  // Other, possibly relevant information..
  char *description;
  char *source;

  // Finally the data...
  double *data;
};

#endif
