#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include "dwm-v1.h"
#include "dwm-groove-region.h"
#include "dwm-debug.h"

void DWMManhattanRegion::Initialize(DWMfloat xcenter, DWMfloat ycenter, 
				    DWMfloat xwidth,  DWMfloat ywidth,
				    DWMfloat depth)
{
  xlow  = xcenter - xwidth / 2.0;
  xhigh = xcenter + xwidth / 2.0;
  ylow  = ycenter - ywidth / 2.0;
  yhigh = ycenter + ywidth / 2.0;
  groove_depth = depth;
}

DWMBoolean DWMManhattanRegion::In_Region(DWMfloat x, DWMfloat y)
{
  if( x >= xlow && x <= xhigh && y >= ylow && y <= yhigh) return DWMTrue;
  return DWMFalse;
}

DWMfloat DWMManhattanRegion::Groove_Depth(DWMfloat x, DWMfloat y)
{
  if( x >= xlow && x <= xhigh && y >= ylow && y <= yhigh) return groove_depth;
  return 0.0;
}

void DWMManhattanRegion::Print()
{
  printf("Manhattan Region: \(%10.2f,%10.2f) to \(%10.2f,%10.2f) depth = %10.2f\n",
	 xlow, ylow, xhigh, yhigh, groove_depth);
}

void DWMManhattanRegion::Print_Diagnostics()
{
  printf("Manhattan Region: \n");
  printf("Name = %s\n", Name());
  printf("xlow  = %f\t ylow  = %f\n", xlow, ylow);
  printf("xhigh = %f\t yhigh = %f\n", xhigh, yhigh);
  printf("groove depth = %f\n", groove_depth);
}
