//
// dwm-grove-region.h
//
// This file provides the class definition for the DWMGrooveRegion 
// class object, and its children which presently include:
// DWMManhattanRegion
//
//
// DWMGrooveRegion is an abstract class designed to serve as a base class 
// for child objects 
// that store information about grooved regions, such as the region that 
// is grooved, the depth of the groove, weather the specified depth is 
// absolute or relative, and the type of groove bounandries.
//
// DWMGrooveRegion and its children support the following operations:
//     1) various constructors. see below.
//     2) DWMBoolean In_Region(DWMfloat,DWMfloat) which returns DWMTrue if
//        the argument pair x,y is in the grooved region.
//     3) DWMfloat Groove_Depth(x, y) which returns the groove depth at the 
//        point x,y.
//     4) DWMGrooveAssociation Groove_Association() which returns 
//        Relative or Absolute
//     5) void Set_Groove_Association( new_value ). 
//        sets the groove association.
//

#ifndef _DWM_GROOVE_REGION_H
#define _DWM_GROOVE_REGION_H

#include "dwm-object.h"

// Presently the absolute - relative distinction is ignored. 9/27/94 gp.
enum DWMGrooveAssociation { Absolute, Relative }; 

class DWMGrooveRegion : public DWMObject {
 public:
  DWMGrooveRegion() { groove_depth = 0.0;  groove_association = Absolute; }
  DWMGrooveRegion(char *groove_name) : DWMObject(groove_name) 
    { groove_depth = 0.0; groove_association = Absolute; }
  ~DWMGrooveRegion() {}

  virtual DWMBoolean In_Region(DWMfloat,DWMfloat) = 0;
  virtual DWMfloat Groove_Depth(DWMfloat,DWMfloat) { return groove_depth; }

  DWMGrooveAssociation Groove_Association() { return groove_association; }
  void Set_Groove_Association( DWMGrooveAssociation new_value )
    { groove_association = new_value; }

 protected:
  DWMfloat groove_depth;
  DWMGrooveAssociation groove_association;
};

class DWMManhattanRegion : public DWMGrooveRegion {
 public:
  DWMManhattanRegion() { Initialize(0.0, 0.0, 0.0, 0.0, 0.0); }
  DWMManhattanRegion(char *groove_name) : DWMGrooveRegion(groove_name)
    { Initialize(0.0, 0.0, 0.0, 0.0, 0.0); }
  DWMManhattanRegion(DWMfloat x1, DWMfloat y1, 
		     DWMfloat x2, DWMfloat y2, 
		     DWMfloat depth)
    { Initialize(x1, y1, x2, y2, depth); }
  ~DWMManhattanRegion() {}

  virtual DWMClassType Class_Type() const { return DWMManhattanRegionClass; }
  virtual char *Class_Name() const { return "DWMManhattanRegionClass"; }

  void Initialize(DWMfloat, DWMfloat, DWMfloat, DWMfloat, DWMfloat);

  virtual DWMBoolean In_Region(DWMfloat,DWMfloat);
  virtual DWMfloat Groove_Depth(DWMfloat,DWMfloat);

  void Print();
  void Print_Diagnostics();

 private:
  DWMfloat xlow, ylow, xhigh, yhigh;

};


#endif
