/*
 * dwm-keyword.h
 *
 * This is a non-abstract class that is ued to store keyword-value
 * information that will be used to processes commands from the lexer.
 * The keyword will be stored in the name area of the class, and the
 * value will be stored in one of two variables depending on the type.
 * No automatic type conversions will take place
 *
 */ 

#ifndef _DWM_KEYWORD_H
#define _DWM_KEYWORD_H

#include <stdio.h>
#include "dwm-object.h"

class DWMKeyword : public DWMObject {
 public:
  DWMKeyword() { initialize(); }
  DWMKeyword(char *name) : DWMObject(name) { initialize(); }

  DWMKeyword(char *name, int initial_i_value) : DWMObject(name) 
    { initialize(); i_value = initial_i_value; }

  DWMKeyword(char *name, double initial_d_value) : DWMObject(name) 
    { initialize(); d_value = initial_d_value; }

  DWMKeyword(char *name, char *initial_c_value) : DWMObject(name) 
    { initialize(); 
      c_value = new char [strlen(initial_c_value+1)];
      strcpy(c_value, initial_c_value);
    }

  ~DWMKeyword() { delete c_value; }

  virtual DWMClassType Class_Type() const { return DWMKeywordClass; }
  virtual char *Class_Name() const { return "DWMKeywordClass"; }

  int    Ivalue() const { return i_value; }
  double Dvalue() const { return d_value; }
  char  *Cvalue() const { return c_value; }

  void Set_Value(int new_i_value) { reset(); i_value = new_i_value; }
  void Set_Value(double new_d_value) 
    { reset(); d_value = new_d_value; }

  void Set_Value(char *new_c_value) 
    { reset();
      c_value = new char [strlen(new_c_value+1)];
      strcpy(c_value, new_c_value);
    }

  virtual void Print_Diagnostics() const 
    {
      printf("DWMKeyword object: name = %s, i_value = %d, d_value = %f, c_value = %s\n", name, i_value, d_value, c_value);
    }

 private:
  int    i_value;
  double d_value;
  char   *c_value;

  unsigned keyword_id;

  void initialize() { i_value = 0; d_value = 0.0; c_value = 0; keyword_id = 0; }
  void reset() { i_value = 0; d_value = 0.0; delete c_value; c_value = 0; keyword_id = 0; }

};

#endif


