#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>

#include "dwm-object.h"
#include "dwm-list.h"
#include "dwm-debug.h"

void DWMList::Add( DWMObject & to_add ) 
{
  ListElement *new_element = new ListElement( &to_add, head->next);
  assert( new_element != 0 );
  head->next = new_element;
  number_of_elements++;
}


void DWMList::Detach( DWMObject &to_detach )
{
  ListElement *ptr;
  ListElement *cursor;

  cursor = find_predecessor( to_detach );

  ptr = cursor->next; // this is the cell to delete
  
  if(ptr != tail) {  // but dont delete if it we are at the end of the list!
                     // In this case, we dont have the element that was specified.
                     // Just return without any action...

    // update the pointers...
    cursor->next = cursor->next->next;

    // do we delete the data element ? Consult owns_list_elements...
    if(owns_list_elements == DWMTrue) delete ptr->data;
    
    // Finally, delete the ListElement itself...
    delete ptr;
    
    // and dont forget the counter...
    number_of_elements--;

  }
}

void DWMList::Flush()
{
  ListElement *cursor, *ptr;

  for(cursor = head->next; cursor!= tail; cursor=ptr) {

    ptr = cursor->next;

    // do we delete the data element ? Consult owns_list_elements...

    if(owns_list_elements == DWMTrue) delete cursor->data;
    
    delete cursor;
  }

  head->next = tail;
  number_of_elements = 0;
}

DWMList::ListElement * DWMList::find_predecessor( DWMObject &object)
{
  tail->data = (DWMObject *)&object;
  ListElement *cursor = head;

  while( &object != cursor->next->data )
    cursor = cursor->next;

  tail->data = 0;
  return cursor;
}

DWMObject *DWMList::Nth(unsigned object_offset) const
{
  // First check to see if the specified index is within range.
  // If not, the return DWMNoObject.....

  if( object_offset < 0 || object_offset >= number_of_elements )
    return( &DWMNoObject );

  // Spacified range is ok. Dig up the data...
  unsigned i;
  ListElement *cursor = head;
  cursor = cursor->next;

  for(i=0; i < object_offset; i++)
    cursor=cursor->next;

  return(cursor->data);
}


DWMObject *DWMList::Contains_Object( DWMObject &search_object ) const
{  
  ListElement *cursor = head;

  while((cursor->next->data != 0) &&
	(search_object != cursor->next->data))
    cursor = cursor -> next;

  if(cursor->next == tail) return( &DWMNoObject );
  else return( cursor->next->data );
}

DWMObject *DWMList::Contains_Named_Object( char *search_name) const
{
  char *t;
  ListElement *cursor = head;

  while(cursor->next->data != 0 &&
	(t = cursor->next->data->Name()) != 0 &&
	strcmp(search_name, t) != 0)
    cursor = cursor -> next;

  if(cursor->next == tail) return( &DWMNoObject );
  else return( cursor->next->data );
}
 
void DWMList::Print_Diagnostics() const 
{
  int i;
  ListElement *cursor = head;

  printf("=== List Diagnostics ===\n");
  printf("This list is named %s, and contains %d elements.\n", name, number_of_elements);
  printf("List Contents:\n");
  
  for(i=0; i< number_of_elements; i++) {
    printf("\t%4d>", i);
    cursor->next->data->Print_Diagnostics();
    cursor = cursor->next;
  }
}
