//
// dwm-list.h
//
// This file provides the class definition for the DWMList class object.
// This object is designed to store other objects (derived from the 
// DWMObject class) as a linked list. Functions are provided to add,
// remove, and lookup elements.  Also, the list can be flushed.  In this 
// case, all elements are removed from the list.
//
//
//

#ifndef _DWM_LIST_H
#define _DWM_LIST_H

#include "dwm-object.h"

class DWMList : public DWMObject {
 public:
  // default creator...
  DWMList() : 
    head_entry(0, &tail_entry),
    tail_entry(0, &tail_entry),
    head(&head_entry),
    tail(&tail_entry),
    number_of_elements(0),
    owns_list_elements(DWMFalse) {}

  // creator that accepts a char* name
  DWMList(char *list_name) : 
    DWMObject(list_name), 
    head_entry(0, &tail_entry),
    tail_entry(0, &tail_entry),
    head(&head_entry),
    tail(&tail_entry),
    number_of_elements(0),
    owns_list_elements(DWMFalse) {}

  virtual ~DWMList() { Flush(); }

  virtual DWMClassType Class_Type() const { return DWMListClass; }
  virtual char *Class_Name() const { return "DWMListClass"; }

  void Add( DWMObject &);
  void Detach( DWMObject &);
  void Flush();
  unsigned Is_Empty() const { return number_of_elements == 0; }
  unsigned Number_Of_Elements() const { return number_of_elements; }
  DWMObject *Nth( unsigned ) const;
  DWMObject *Contains_Object( DWMObject & ) const;
  DWMObject *Contains_Named_Object( char *) const;

  DWMBoolean &Owns_List_Elements() { return owns_list_elements; }

  virtual void Print_Diagnostics() const;
  
 protected:

  class ListElement {
  public:
    ListElement( DWMObject *object, ListElement *n = 0) {
      data = object;
      next = n;
    }
  private:
    DWMObject *data;
    ListElement *next;
    friend class DWMList;
    friend class DWMFlickList;
  };

  ListElement *find_predecessor( DWMObject &);

  ListElement *head;
  ListElement *tail;
  ListElement head_entry;
  ListElement tail_entry;
  unsigned number_of_elements;
  DWMBoolean owns_list_elements;
};

#endif

