//
// dwm-timing-obj.h
//
// This file provides the class definition for the DWMTimingObject 
// class object.
//
// This object is designed to store timing data that represents 
// a current pules applied to a particular conductor. The user interface
// functions are:
//
//
// George Patterson
// Aug 19 1994 -- started
//

#ifndef _DWM_TIMING_OBJECT_H
#define _DWM_TIMING_OBJECT_H

typedef double DWMfloat;

#include "dwm-object.h"

class DWMTimingObject : public DWMObject {

 public:
  /* here are the constructors.  I have provided one with no arguments,
     and others that accept all of the timing data. */

  DWMTimingObject() :
    bias_current(0.0), pulse_current(0.0),
    rise_time(10.0), fall_time(10.0),
    pulse_width(10.0), pulse_offset(0.0),
    cycle_time(0.0) 
      { update_internals(); }

  DWMTimingObject(char *name, DWMfloat bcurrent = 0.0, DWMfloat pcurrent = 0.0,
		  DWMfloat rtime = 10.0, DWMfloat ftime = 10.0,
		  DWMfloat pwidth = 10.0, DWMfloat poffset = 10.0,
		  DWMfloat ctime = 0.0) : 
		    DWMObject(name), 
		    bias_current(bcurrent), pulse_current(pcurrent),
		    rise_time(rtime), fall_time(ftime),
		    pulse_width(pwidth), pulse_offset(poffset),
		    cycle_time(ctime) 
		      { update_internals(); }

  ~DWMTimingObject() {}

  /* First the necessary functions to make this a concrete object */

  virtual DWMClassType Class_Type() const { return DWMTimingObjectClass; }
  virtual char *Class_Name() const { return "DWMTimingObjectClass"; } 

  DWMfloat Bias_Current()  const { return bias_current; }
  DWMfloat Pulse_Current() const { return pulse_current; }
  DWMfloat Rise_Time()     const { return rise_time; }
  DWMfloat Fall_Time()     const { return fall_time; }
  DWMfloat Pulse_Width()   const { return pulse_width; }
  DWMfloat Pulse_Offset()  const { return pulse_offset; }
  DWMfloat Cycle_Time()    const { return cycle_time; }

  void Set_Bias_Current( DWMfloat new_value )  {  bias_current = new_value; }
  void Set_Pulse_Current( DWMfloat new_value ) {  pulse_current = new_value; }
  void Set_Rise_Time( DWMfloat new_value )     {  rise_time = new_value; update_internals(); }
  void Set_Fall_Time( DWMfloat new_value )     {  fall_time = new_value; update_internals(); }
  void Set_Pulse_Width( DWMfloat new_value )   {  pulse_width = new_value; update_internals(); }
  void Set_Pulse_Offset( DWMfloat new_value )  {  pulse_offset = new_value; update_internals(); }
  void Set_Cycle_Time( DWMfloat new_value )    {  cycle_time = new_value; update_internals(); }

  virtual void Print_Diagnostics() const;

  DWMfloat TScale( DWMfloat time) const;

 private:

  /**** 
     Remember: all currents are measured in mA
     all times are measured in ns.     
     ****/

  DWMfloat bias_current, pulse_current;
  DWMfloat rise_time, fall_time;
  DWMfloat pulse_width, pulse_offset;
  DWMfloat cycle_time;


  DWMfloat t1, t2, t3, t4;
  DWMfloat rise_time_slope, fall_time_slope;

  void update_internals() {
    t1 = 0.0;
    t2 = rise_time;
    t3 = t2 + pulse_width;
    t4 = t3 + fall_time;

    rise_time_slope = 1.0 / rise_time;
    fall_time_slope = 1.0 / fall_time;
  }

};


#endif
