class DWMFlickList;
class Vector;

#define PI    M_PI
#define TWOPI 6.28318530717958647692
#define PIOVERTWO 1.57079632679489661923
#define RADTODEG  57.29577951

typedef double DWMfloat;
typedef double DWMFloat;

struct InputMaterialParameters {
  double four_pi_ms;
  double characteristic_length;
  double quality_factor;
  double alpha;
  double wall_mobility;
  double maximum_wall_velocity;
  double material_thickness;
  double coercivity;
};

struct MaterialParameters {
  double four_pi_ms;
//  double characteristic_length;
  double quality_factor;
  double alpha;
  double wall_mobility;
  double maximum_wall_velocity;
  double maximum_wall_force;
  double material_thickness;
  double coercivity;

  DWMfloat distance_scale_factor;
  DWMfloat time_scale_factor;
  DWMfloat magnetic_field_scale_factor;
};

struct InputRunParameters {
  double time_step;
  double total_time_requested;
  double state_save_interval;
  char base_name[256];
};

struct RunParameters {
  double time_step;
  double total_time_requested;
  double state_save_interval;
  double time;
  int    step;
  FILE *debug_fp;
  FILE *state_fp;
  FILE *summary_fp;
  char base_name[256];
  int debug;
};

void ludcmp(double **a,int n,int *indx,double *d);
void lubksb(double **a,int n,int *indx,double *b);

void nrerror(char *error_text);
double *vector(int nl, int nh);
void free_vector(double *v, int nl, int nh);

void free_matrix(double **m,int nrl,int nrh,int ncl,int nch);
double **matrix(int nrl,int nrh,int ncl,int nch);

DWMfloat Normalize_Angle(DWMfloat);
DWMfloat Normalize_Angle(DWMfloat, DWMfloat);


/* These are the main sentence processing routines from dwm-main.cc */

void process_load_sentence(char *);
void process_load_state_sentence(char *);
void process_load_field_sentence(char *, char *, DWMFlickList *);
void process_load_ascii_field_sentence(char *, char *, DWMFlickList *);
void process_create_bubble_sentence(char *, DWMFlickList *);
void process_create_tsequence_sentence(char *, char *, DWMFlickList *);
void process_create_groove_sentence(char *,double, double, double, double, double, DWMFlickList *);
void process_simple_set_sentence(char *, DWMfloat);
void process_simple_set_sentence(char *, char *);
void process_delete_sentence(char *, char *);
void process_run_sentence(DWMfloat foo = -1.0);
void process_reset_sentence();
void process_print_sentence(char *print_arg_1 = 0 , char *print_arg_2 = 0);
void process_quit_sentence();
void process_comment_sentence(DWMFlickList *);
void process_write_field_sentence(char *, DWMFloat,  DWMFloat,  DWMFloat,  DWMFloat,  DWMFloat,  DWMFloat);
void process_write_object_sentence(char *, char *);

/* Additional routines, called from the main sentence processing routines. */

void print_parameters();
void print_domain_walls();
void print_external_fields();
void reset_file_pointers();
DWMfloat Unscaled_Material_Thickness(DWMfloat, DWMfloat);
DWMfloat Material_Thickness(Vector &location);
DWMfloat Material_Thickness(DWMfloat, DWMfloat);
void calculate_internal_parameters(void);
void reset_domain_walls();
void set_all_timing_object_currents(DWMfloat);
void setup_output_file(FILE *);

/* Routines that calculate the effective groove field */
double groove_component_z(double, double, double);
double groove_field_z(double , double , double , double , double , double );
double average_groove_field_z(double, double , double , double , double , double, double);
double average_ridge_field_z(double, double , double , double , double , double, double);

void ml_optimize(double, double, double, double, double);
void groove_characterization();


// routines from dwm-utilities.cc
char *strip_white(char *);
char *extract_token(char *, char *);
