/*
   vector1.cc

   This file contains a simple class library to to vector
   algebra in double float precision.


   Started with TURBO C on IBM PC Clone 
   Ported to SGI.


   George Patterson

*/



/* #include <iostream.h> */

#include <stdio.h>
#include <assert.h>
#include <math.h>


#include "vector-1.h"


/* First a constructor that initializes the vector with a number of elements */

Vector::Vector(unsigned d)
{
	dimensions=d;
	representation=0;
	components = new double[d];
	for(int i=0; i < d; i++) *(components+i) = 0.0;
}


// Here we provide a copy constructor for simple initialization and return
// of objects on the stack.

Vector::Vector(const Vector &source)
{
	dimensions = source.dimensions;
	representation = source.representation;
	components = new double [dimensions];
	for( int i=0; i < dimensions; i++)
	  *(components+i) = *(source.components+i);
}


Vector& Vector::operator= (const Vector& s)
{

  /* first see if there is any memory associates with the vector */
  /* if not, then allocate some..... */

  if(dimensions == 0) {
    components = new double [s.dimensions];
    dimensions = s.dimensions;
    }

  /* Reallocate memory if there is a size difference in the vectors */

  if(dimensions!= s.dimensions) {
    delete [] components;
    components = new double [s.dimensions];
    dimensions = s.dimensions;
  }
  

  /* Ok -- now copy over the components. The memory is all set. */
  
  representation = s.representation;
  for(int i=0; i< dimensions; i++)  *(components+i) = *(s.components+i);
  return *this;
}


void Vector::Stats()
{
	printf("Vector stats: dimensions = %d, representation = %d, elements = (",
	       dimensions, representation);
	
	for(int i=0; i< dimensions; i++)
	  printf(" %f", *(components + i));
	
	printf(" )\n");
}


void Vector::Print()
{
	printf("(");

	if(dimensions>0) {
		for(int i=0; i<(dimensions-1); i++)
			printf(" %f,",*(components+i));
		printf(" %f",*(components+(dimensions-1)));
		}

	printf(" )\n");
}


void Vector::Set_Component(int position, double value)
{
	assert(position>=0 && position<dimensions);
	*(components+position) = value;
}


double Vector::Get_Component(int position) const
{
	assert(position>=0 && position<dimensions);
	return( *(components+position) );
}

void Vector::Scale_Component(int position, double scale_factor)
{
  assert(position>=0 && position<dimensions);
  *(components+position) *= scale_factor;
}


void Vector::Normalize()
{	
	double magnitude=0.0;
	double val=0.0;
	for(int i=0; i< dimensions; i++) {
		val = *(components+i);
		magnitude += val * val;
	}

	if(magnitude != 0.0) magnitude = 1.0/sqrt(magnitude);
	else magnitude = 1.0;

	for(i=0; i< dimensions; i++) {
		*(components+i) *= magnitude;
	}
}


double Vector::Magnitude()
{
	double m = 0.0; double v=0.0;
	if(dimensions!=0) {
		for(int i=0; i<dimensions; i++) {
			v = *(components+i);
			m += v*v;
		}
		m = sqrt(m);
	}
	return(m);
}


void Vector::Scale(double scale_factor)
{
	for(int i=0; i<dimensions; i++)
	  *(components+i) *= scale_factor;
}


Vector operator+(const Vector& v1, const Vector& v2)
{
	assert(v1.dimensions == v2.dimensions);
	assert(v1.representation == v2.representation);

	Vector result=v1;

	for(int i=0; i<result.dimensions; i++) {
		*(result.components+i) = *(v1.components+i) + 
		  *(v2.components+i);
	}
	return(result);
}


Vector operator-(const Vector& v1, const Vector& v2)
{       
  assert(v1.dimensions == v2.dimensions);
  assert(v1.representation == v2.representation);
  
  Vector result = v1;
  for(int i=0; i<v1.dimensions; i++) {
    *(result.components+i) = *(v1.components+i) - *(v2.components+i);
  }
  return(result);
}


double DotProduct(const Vector& v1, const Vector& v2)
{
	assert(v1.dimensions == v2.dimensions);
	double result = 0.0;
	for(int i=0; i<v1.dimensions; i++)
	  result += *(v1.components+i) * *(v2.components+i);
	return(result);
}


Vector CrossProduct(const Vector &v1, const Vector& v2)
{
	/* right now this only works for 3 dimensional cartesian vectors */

	assert(v1.dimensions == 3 && v2.dimensions == 3);

	Vector result(3);


	result.Set_Component(0,
			    v1.Get_Component(1) * v2.Get_Component(2) -
			    v1.Get_Component(2) * v2.Get_Component(1));

        result.Set_Component(1,
			    v1.Get_Component(2) * v2.Get_Component(0) -
			    v1.Get_Component(0) * v2.Get_Component(2));

	result.Set_Component(2,
			    v1.Get_Component(0) * v2.Get_Component(1) -
			    v1.Get_Component(1) * v2.Get_Component(0));

	return(result);
}


