//
// dwm-object.h
//
//
//

#ifndef _DWM_OBJECT_H
#define _DWM_OBJECT_H

#include <stdio.h>
#include <string.h>

// First we define an enumeration type which describes the classes
// that we plan to work with in the DWM model.  This type will be used
// to identify the various object types  if we loose track of
// them, for example, in a bag or list.
// 
// To add new objects to the DWM system, you must add a type to the
// DWMClassType enumeration, and also derive the object from the DWMObject
// class that is given below.

enum DWMClassType {
  DWMDomainWallClass, 
  DWMWallPointClass,
  DWMListClass, 
  DWMErrorClass,
  DWMDebugClass,
  DWMTimingObjectClass,
  DWMDiscreteFieldClass,
  DWMKeywordClass,
  DWMGrooveFieldClass,
  DWMManhattanRegionClass,
  DWMSegmentClass
};

enum DWMBoolean {
  DWMTrue,
  DWMFalse
};



// The DWMObject class is an abstract base class (or super class) 
// for all the DWM simulation objects.
//
// Please derive all new object from this object, or one of its derived 
// classes.  Also, it is preferable to declare this class a public base
// class of any future classes to exploit C++ polymorphism.

class DWMObject {

 public:
  
  DWMObject() { name = 0; }
  DWMObject(char *object_name) { 
    name = new char [strlen(object_name) + 1];
    strcpy(name, object_name);
  }

  virtual ~DWMObject() { delete name; }

  virtual DWMClassType Class_Type() const = 0;
  virtual char *Class_Name() const = 0;

  char *Name() const { return name; }
  void Rename(char *new_name) { 
    delete name;
    name = new char [strlen(new_name) + 1];
    strcpy(name, new_name);
  }
  
  virtual void Print() const {};
  virtual void Print_Diagnostics() const {};
  void Write(FILE *fp) { fprintf(fp,"The Write function has not been implemented for the object named %s.\n",Name()); }
  //void Read(FILE *fp) {};
  void Write_Object_Description(FILE *fp) { fprintf(fp,"The Write_Object_Description function has not been implemented for the object named %s.\n",Name()); }

 protected:
  char *name;
};

// Next, we provide some declarations that relate (generally) to
// DWMObjects....

int operator== (DWMObject &item1, DWMObject &item2);
int operator!= (DWMObject &item1, DWMObject &item2);

int operator== (DWMObject *item1, DWMObject &item2);
int operator!= (DWMObject *item1, DWMObject &item2);

int operator== (DWMObject &item1, DWMObject *item2);
int operator!= (DWMObject &item1, DWMObject *item2);

#endif

